#!/usr/bin/perl
# This program is copyright Jonathan Feinberg 1999.

# This program is distributed under the same terms as infobot.

# Jonathan Feinberg   
# jdf@pobox.com
# http://pobox.com/~jdf/

# Version 1.2
# First public release.
# 
# 2001-05-27 Rich Lafferty <rich@alcor.concordia.ca>
#     - new layout -- use tokens in translated text to find begin/end\
#     - optionally grok Unicode
# ????-??-?? Tim@Rikers.org:
#     - handle new URL and layout
# 
package babel;
use strict;

my ($no_babel, $no_babel_unicode);

BEGIN {
    eval "use URI::Escape";    # utility functions for encoding the 
    if ($@) { $no_babel++};    # babelfish request
    eval "use LWP::UserAgent";
    if ($@) { $no_babel++};
    eval "use Unicode::String";
    if ($@) { $no_babel_unicode++};
}

BEGIN {
  # Translate some feasible abbreviations into the ones babelfish
  # expects.
    use vars qw!%lang_code $lang_regex %marker!;
    %lang_code = (
                sp => 'es',
                es => 'es',
                fr => 'fr',
                ge => 'de',
                de => 'de',
                gr => 'de',
                it => 'it',
                po => 'pt',
                pt => 'pt',
                en => 'en'
               );
  
  # Here's how we recognize the language you're asking for.  It looks
  # like RTSL saves you a few keystrokes in #perl, huh?
  $lang_regex = join '|', keys %lang_code;
  
  # Markers to delimit response
  %marker = ( BEGIN => "__INFOBOT_BEGIN", END => "__INFOBOT_END");
}


sub forking_babelfish {
    return '' if $no_babel;
   my ($direction, $lang, $phrase, $callback) = @_;
   $SIG{CHLD} = 'IGNORE';
   my $pid = eval { fork() };   # catch non-forking OSes and other errors
   return if $pid;              # parent does nothing
   $callback->(babelfish($direction, $lang, $phrase));
   exit 0 if defined $pid;      # child exits, non-forking OS returns
}

sub babelfish {
    return '' if $no_babel;
  my ($direction, $lang, $phrase) = @_;
  
  $lang = $lang_code{$lang};

  my $ua = new LWP::UserAgent;
  $ua->timeout(5);

  my $req =  
    HTTP::Request->new('POST', 'http://babel.altavista.com/tr');
  $req->content_type('application/x-www-form-urlencoded');

  my $tolang = "en_$lang";
  my $toenglish = "${lang}_en";

  if ($direction eq 'to') {
    return translate($phrase, $tolang, $req, $ua);
  }
  elsif ($direction eq 'from') {
    return translate($phrase, $toenglish, $req, $ua);
  }

  my $last_english = $phrase;
  my $last_lang;
  my %results = ();
  my $i = 0;
  while ($i++ < 7) {
    last if $results{$phrase}++;
    $last_lang = $phrase = translate($phrase, $tolang, $req, $ua);
    last if $results{$phrase}++;
    $last_english = $phrase = translate($phrase, $toenglish, $req, $ua);
  }
  return $last_english;
}


sub translate {
  return '' if $no_babel;
    
  my ($phrase, $languagepair, $req, $ua) = @_;
  &main::status ("babel: $languagepair translation of '$phrase'");
  
  # By adding a marker to either end of the query text, we can avoid
  # relying on Altavista's HTML, and just take out the phrase in question. 
  # egrep++ for the idea
  my $urltext = uri_escape(".$marker{BEGIN}: $phrase : .$marker{END}");
  
  $req->content("urltext=$urltext&lp=$languagepair&tt=urltext");
  
  my $res = $ua->request($req);

  if ($res->is_success) {
      my $html = $res->content;
      
      # We can pull out results by taking the bit between the markers
      # we added above. For the end marker, if the original phrase ended
      # in punctuation, then we get the punctuation mark followed by
      # our end marker; if it *didn't* end in punctuation, we get ':'
      # followed by our end marker. The start marker always has the trailing
      # ':'.
      my ($translated) = ($html =~ m/$marker{BEGIN}:\s*(.*?)\s+$marker{END}/sx);

      $translated =~ s/\n/ /g;
      $translated =~ s/:?\s*$//;
      
      if ($no_babel_unicode) {
      	&main::status("Warning: Babel requires Unicode::String to handle non-ASCII characters");
      	return $translated;
      }
      else {
      	# halfjack++ for figuring this out for me :-)
      	my $utf = Unicode::String::utf8($translated); 
      	return $utf->latin1;    
      }

  } else {
      return ":("; # failure 
  }
}

if (0) {
    if (-t STDIN) {
        my $result = babel::babelfish('to','sp','hello world');
        $result =~ s/; /\n/g;
        print "Babelfish says: $result\n";
    }
}

1;
